﻿using System.Collections.Generic;

using Microsoft.Msagl.Core.Layout;
using Microsoft.Msagl.Core.Routing;

using DevExpress.Diagram.Core;
using DevExpress.Diagram.Core.Layout;

namespace TestLibrary
{
    /// <summary>
    /// 그래프 레이아웃
    /// </summary>
    public class GraphLayout
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 레이아웃 계산기 - LayoutCalculator

        /// <summary>
        /// 레이아웃 계산기
        /// </summary>
        protected ILayoutCalculator LayoutCalculator { get; set; }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 기하 그래프 - GeometryGraph

        /// <summary>
        /// 기하 그래프
        /// </summary>
        private GeometryGraph GeometryGraph { get; set; }

        #endregion
        #region 라우팅 모드 - RoutingMode

        /// <summary>
        /// 라우팅 모드
        /// </summary>
        private EdgeRoutingMode RoutingMode
        {
            get
            {
                return LayoutCalculator.LayoutAlgorithmSettings.EdgeRoutingSettings.EdgeRoutingMode;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - GraphLayout(layoutCalculator)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="layoutCalculator">레이아웃 계산기</param>
        public GraphLayout(ILayoutCalculator layoutCalculator)
        {
            LayoutCalculator = layoutCalculator;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 그래프 노드 위치 레이아웃 다시 설정하기 - RelayoutGraphNodePosition(graph)

        /// <summary>
        /// 그래프 노드 위치 레이아웃 다시 설정하기
        /// </summary>
        /// <param name="graph">그래프</param>
        /// <returns>위치 정보 열거 가능 목록</returns>
        public virtual IEnumerable<PositionInfo<IDiagramItem>> RelayoutGraphNodePosition(Graph<IDiagramItem> graph)
        {
            GeometryGraph = MSAGLGeometryGraphHelper.GetGeometryGraph(graph);

            LayoutCalculator.CalculateLayout(GeometryGraph);

            return MSAGLGeometryGraphHelper.GetNodePositionInfo(GeometryGraph);
        }

        #endregion
        #region 다이어그램 커넥터 타입 구하기 - GetDiagramConnectorType()

        /// <summary>
        /// 다이어그램 커넥터 타입 구하기
        /// </summary>
        /// <returns>다이어그램 커넥터 타입</returns>
        public ConnectorType GetDiagramConnectorType()
        {
            return RoutingHelper.GetDiagramConnectorType(RoutingMode);
        }

        #endregion
    }
}