﻿using System;
using System.Collections.Generic;

using Microsoft.Msagl.Core.Geometry.Curves;
using Microsoft.Msagl.Core.Layout;
using Microsoft.Msagl.Core.Routing;
using Microsoft.Msagl.Layout.Layered;
using Microsoft.Msagl.Layout.MDS;

namespace TestLibrary
{
    /// <summary>
    /// 비연결 그래프 레이아웃 계산기
    /// </summary>
    public class DisconnectedGraphsLayoutCalculator : ILayoutCalculator
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 레이아웃 알고리즘 설정 - LayoutAlgorithmSettings

        /// <summary>
        /// 레이아웃 알고리즘 설정
        /// </summary>
        public LayoutAlgorithmSettings LayoutAlgorithmSettings
        {
            get
            {
                return new SugiyamaLayoutSettings()
                {
                    Transformation      = PlaneTransformation.Rotation(Math.PI),
                    EdgeRoutingSettings = { EdgeRoutingMode = EdgeRoutingMode.StraightLine }
                };
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 레이아웃 계산하기 - CalculateLayout(geometryGraph)

        /// <summary>
        /// 레이아웃 계산하기
        /// </summary>
        /// <param name="geometryGraph">기하 그래프</param>
        public void CalculateLayout(GeometryGraph geometryGraph)
        {
            IEnumerable<GeometryGraph> componentEnumerable = GraphConnectedComponents.CreateComponents
            (
                geometryGraph.Nodes,
                geometryGraph.Edges
            );

            SugiyamaLayoutSettings setting = LayoutAlgorithmSettings as SugiyamaLayoutSettings;

            foreach(GeometryGraph component in componentEnumerable)
            {
                LayeredLayout layout = new LayeredLayout(component, setting);

                component.Margins = 100;

                layout.Run();
            }

            MdsGraphLayout.PackGraphs(componentEnumerable, setting);

            geometryGraph.UpdateBoundingBox();
        }

        #endregion
    }
}