﻿using System.Collections.Generic;

using Microsoft.Msagl.Core.Layout;
using Microsoft.Msagl.Core.Routing;
using Microsoft.Msagl.Layout.MDS;
using Microsoft.Msagl.Prototype.Ranking;

namespace TestLibrary
{
    /// <summary>
    /// 랭킹 레이아웃 계산기
    /// </summary>
    public class RankingLayoutCalculator : ILayoutCalculator
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 레이아웃 알고리즘 설정 - LayoutAlgorithmSettings

        /// <summary>
        /// 레이아웃 알고리즘 설정
        /// </summary>
        public LayoutAlgorithmSettings LayoutAlgorithmSettings
        {
            get
            {
                return new RankingLayoutSettings()
                {
                    NodeSeparation      = 30,
                    EdgeRoutingSettings = { EdgeRoutingMode = EdgeRoutingMode.Rectilinear }
                };
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 레이아웃 계산하기 - CalculateLayout(geometryGraph)

        /// <summary>
        /// 레이아웃 계산하기
        /// </summary>
        /// <param name="geometryGraph">기하 그래프</param>
        public void CalculateLayout(GeometryGraph geometryGraph)
        {
            IEnumerable<GeometryGraph> componentEnumerable = GraphConnectedComponents.CreateComponents
            (
                geometryGraph.Nodes,
                geometryGraph.Edges
            );

            RankingLayoutSettings setting = LayoutAlgorithmSettings as RankingLayoutSettings;

            foreach(GeometryGraph component in componentEnumerable)
            {
                RankingLayout layout = new RankingLayout(setting, component);

                component.Margins = 30;

                layout.Run();
            }

            MdsGraphLayout.PackGraphs(componentEnumerable, setting);

            geometryGraph.UpdateBoundingBox();
        }

        #endregion
    }
}